/*
Super Jump And Run Wars
Copyright (C) 2011-2012 Denis Meyer, CallToPower Software

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License version 3, 29 June 2007, as published by the Free Software Foundation.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

var URL = "localhost";
var PORT = 8182;

// misc
var windowWidth = 500;
var windowHeight = 400;
var keyTickRate = 15;

// colors and more
var timeColor = "#D15705";
var nameColor = "#6195ED";
var scoreNameColor = "#3399FF";
var canvasBorder = "#000 2px solid";
var contextFillStyle = "#FFF";
var contextFont = '15px sans-serif';

// sprites/images
var themeFolder = 'supermario'; // pre-defined themes: original, supermario
var spriteFolder = 'img/game/sprites/' + themeFolder + '/';
var playerSprites = [   spriteFolder + 'player1',
                        spriteFolder + 'player2',
                        spriteFolder + 'player3',
                        spriteFolder + 'player4',
                        spriteFolder + 'player5',
                        spriteFolder + 'player1_l', spriteFolder + 'player1_r',
                        spriteFolder + 'player2_l', spriteFolder + 'player2_r',
                        spriteFolder + 'player3_l', spriteFolder + 'player3_r',
                        spriteFolder + 'player4_l', spriteFolder + 'player4_r',
                        spriteFolder + 'player5_l', spriteFolder + 'player5_r'];
var tileSprites = [ spriteFolder + 'tile1',
                    spriteFolder + 'tile2',
                    spriteFolder + 'tile3'];
var backgroundImg = 'img/game/background1';

// sounds
var path_soundBackgroundIntro = "audio/smb.ogg";
var path_soundBackground = "audio/smb_loop.ogg";
var path_soundJump = "audio/jump.wav";
var path_soundKill = "audio/kill.wav";
var path_soundHit = "audio/hit.wav";

// image suffixes
var playerSpriteSuffix = ".png";
var tileSpriteSuffix = ".png";
var backgroundSuffix = ".png";
var playerLeftStr = "_l";
var playerRightStr = "_r";

// ids
var sound_id_bg = "sound_bg";
var sound_id_jump = "sound_jump";
var sound_id_kill = "sound_kill";
var sound_id_hit = "sound_hit";
var id_mainContainer = "mainContainer";
var id_main = "main";
var id_joinGame = "joinGame";
var id_div_loading = "div_loading";
var id_chat_window = "chat_window";
var id_runLeft = "runLeft";
var id_runRight = "runRight";
var id_jump = "jump";
var id_div_canvas = "div_canvas";
var id_div_controls = "div_controls";
var id_clientCount = "clientCount";
var id_scoreTable = "scoreTable";
var id_canvas = "canvas";
var id_key_space = "key_space";
var id_key_arrowLeft = "key_arrowLeft";
var id_key_arrowRight = "key_arrowRight";
var id_newMessage = "newMessage";

// strings
var str_errorBrowserSupportHtml5 = "Your browser does not support html5 canvas.";
var str_firstStart = "<a href=\"#\" id='" + id_joinGame + "' class=\"button_joinGame\">Join game</a>";
var str_players = "Players";
var str_score = "Score";
var str_connected = "connected";
var str_disconnected = "disconnected";
var str_killed = "killed";
var str_welcome = "Welcome to Super Jump'n'Run Wars!";
var str_chatInit = "Feel free to chat!";
var str_loading = "Loading game...";
var str_runLeft = "Run left";
var str_runRight = "Run right";
var str_jump = "Jump";

/**
 * formats a message
 * @param timeString
 *      the time
 * @param name
 *      name of the player
 * @param msg
 *      the message
 * @return
 *      a formatted message
 */
function formatMsg(timeString, name, msg) {
    return "<br />&nbsp;<span style='color:" + timeColor + ";'>" + timeString + "</span> - <span style='color:" + nameColor + ";'>" + name + "</span>: " + msg;
}

/**
 * returns a time string
 * @return
 *      a time string
 */
function getTimeString() {
    var date = new Date();
    var h = date.getHours();
    var m = date.getMinutes();
    var s = date.getSeconds();
    return ((h < 10) ? ("0" + h) : h) + ":" + ((m < 10) ? ("0" + m) : m) + ":" + ((s < 10) ? ("0" + s) : s);
}

/**
 * checks whether num is in [lower, upper]
 * @param lower
 *      lower bound
 * @param upper
 *      upper bound
 * @return
 *      true when num is in [lower, upper], false else
 */
function inInterval(lower, upper, num) {
    return ((num >= lower) && (num <= upper));
}

/**
 * returns a random number in between [min, max]
 * @param min
 *      min value
 * @param max
 *      max value
 * @return
 *      a random number in between [min, max]
 **/
function getRandom(min, max) { 
    if (min == max) {
        return min;
    } else if (min > max) {
        var tmp = max;
        max = min;
        min = max;
    }
    return (min + parseInt(Math.random() * (max - min + 1)));
}

(function ($) {
    var ASSET_MANAGER = new AssetManager();
    var gameEngine, canvas, context, background;

    var socket, id;
    var firstRun = true;

    var keySpace = false,
        keyArrowLeft = false,
        keyArrowRight = false;

    var msgPlayers, msgTiles;

    var bgs, bgs_loop;
    var bgsPL1 = false,
        bgsPL2 = false;

    /**
     * queue downloads in the asset manager
     */
    function queueDownloads() {
        ASSET_MANAGER.queueDownload(backgroundImg + backgroundSuffix);
        for (var i = 0; i < playerSprites.length; ++i) {
            ASSET_MANAGER.queueDownload(playerSprites[i] + playerSpriteSuffix);
        }
        for (var i = 0; i < tileSprites.length; ++i) {
            ASSET_MANAGER.queueDownload(tileSprites[i] + tileSpriteSuffix);
        }
    }

    /**
     * download queued downloads
     */
    function downloadAll() {
        ASSET_MANAGER.downloadAll(function () {
            // prepare canvas
            canvas = document.getElementById(id_canvas);
            canvas.width = windowWidth;
            canvas.height = windowHeight;
            canvas.style.border = canvasBorder;
            // check html5 canvas support
            if (canvas.getContext) {
                context = canvas.getContext('2d');

                context.fillStyle = contextFillStyle;
                context.font = contextFont;
                context.textBaseline = 'top';
                context.fillText(str_loading, canvas.width / 2 - str_loading.length * 2.5, canvas.height / 2);

                initSockets();
                initBGSound();
            } else {
                $('#' + id_mainContainer + ', #' + id_div_loading).html(str_errorBrowserSupportHtml5).show();
            }
        });
    }
    
    /**
     * sets up the background sound loop
     */
    function setupBgsLoopListener() {
        var bgs_checked = $('#' + sound_id_bg).is(':checked');
        if (bgs_loop) {
            if (typeof bgs_loop.sound.loop == 'boolean') {
                bgs_loop.sound.loop = true;
            } else {
                bgs_loop.sound.addEventListener('ended', function () {
                    this.currentTime = 0;
                    this.play();
                }, false);
            }
            if(bgs_checked) {
                bgs_loop.play();
            }
        }
    }

    /**
     * initializes the background sounds, plays the intro and loops the loop
     */
    function initBGSound() {
        var bgs_checked = $('#' + sound_id_bg).is(':checked');
        bgs = new Sound(path_soundBackgroundIntro);
        if (bgs) {
            bgs.sound.addEventListener('canplaythrough', function () {
                if (!bgsPL1) {
                    bgsPL1 = true;
                    bgs_loop = new Sound(path_soundBackground);
                    bgs_loop.sound.addEventListener('canplaythrough', function () {
                        if (!bgsPL2) {
                            bgsPL2 = true;
                            if(bgs_checked) {
                                bgs.play();
                            }
                            bgs.sound.addEventListener('ended', function () {
                                setupBgsLoopListener();
                            });
                        }
                    });
                }
            }, false);
        }
    }

    /**
     * initializes the socket connection
     */
    function initSockets() {
        socket = io.connect(URL, {
            port: PORT
        });
        // socket.on('message', function (msg) {});
        socket.on('tilesUpdate', function (msg) {
            msgTiles = msg.tiles;
            if (!firstRun) {
                updateTiles(msgTiles);
            }
        });
        socket.on('socketId', function (msg) {
            id = msg.id;
            if (firstRun) {
                initGameEngine();
                firstRun = false;
            }
        });
        socket.on('initPlayers', function (msg) {
            updatePlayers(msg.players);
        });
        socket.on('playerUpdate', function (msg) {
            updatePlayer(msg.id, msg.x, msg.y, msg.dl, msg.dr);
        });
        socket.on('chatUpdate', function (msg) {
            $('#' + id_chat_window).prepend(formatMsg(getTimeString(), msg.id.substr(0, 5), msg.msg));
        });
        socket.on('playerConnected', function (msg) {
            setupPlayer(msg.id, msg.x, msg.y, msg.spriteNr);
            if (msg.id != id) {
                gameEngine.setMessage(getTimeString() + ": " + msg.id.substr(0, 5) + " " + str_connected);
            }
            updateScores(msg.scores, msg.activeClients);
        });
        socket.on('playerDisconnected', function (msg) {
            if (msg.id != id) {
                delete gameEngine.players[msg.id];
                gameEngine.setMessage(getTimeString() + ": " + msg.id.substr(0, 5) + " " + str_disconnected);
                updateScores(msg.scores, msg.activeClients);
            }
        });
        socket.on('playerKilled', function (msg) {
            gameEngine.setMessage(getTimeString() + ": " + msg.killer.substr(0, 5) + " " + str_killed + " " + msg.victim.substr(0, 5));
            if (id == msg.victim) {
                var player = gameEngine.getPlayer(id);
                if (player) {
                    player.playHit();
                    player.setX(getRandom(0, canvas.width));
                    player.setY(-getRandom(0, canvas.height));
                }
            }
            updateScores(msg.scores, msg.activeClients);
        });
    }

    /**
     * initializes some click events
     */
    function initClickEvents() {
        $('#' + sound_id_bg).click(function () {
            if ($(this).is(':checked')) {
                setupBgsLoopListener();
            } else {
                if(bgs) {
                    bgs.stop();
                }
                if(bgs_loop) {
                    bgs_loop.sound.pause();
                }
            }
        });
        $('#' + sound_id_bg + ', #' + sound_id_jump + ', #' + sound_id_kill + ', #' + sound_id_hit).click(function () {
            focusCanvas();
        });
    }

    /**
     * initializes the client game engine
     */
    function initGameEngine() {
        gameEngine = new GameEngine(id);
        gameEngine.init(context);

        background = new Background(ASSET_MANAGER.getAsset(backgroundImg + backgroundSuffix));
        gameEngine.setBackground(background);

        $('#' + id_div_loading).hide();
        $('#' + id_runLeft).html(str_runLeft);
        $('#' + id_runRight).html(str_runRight);
        $('#' + id_jump).html(str_jump);
        $('#' + id_mainContainer + ', #' + id_div_canvas + ', #' + id_div_controls).show();

        focusCanvas();
        setKeyEvents();
        gameEngine.start();
        updateTiles(msgTiles);
        initClickEvents();
    }

    /**
     * updates the players
     * @param players
     *      list of all players
     */
    function updatePlayers(players) {
        // update all players but own
        for (player in players) {
            if (!gameEngine.getPlayer(player)) {
                gameEngine.setPlayer(new Player(
                players[player].x, players[player].y, ASSET_MANAGER.getAsset(playerSprites[players[player].spriteNr] + playerSpriteSuffix), ASSET_MANAGER.getAsset(playerSprites[players[player].spriteNr] + playerLeftStr + playerSpriteSuffix), ASSET_MANAGER.getAsset(playerSprites[players[player].spriteNr] + playerRightStr + playerSpriteSuffix)), player);
            } else {
                if (player != id) {
                    var pl = gameEngine.getPlayer(player);
                    pl.setX(players[player].x);
                    pl.setY(players[player].y);
                }
            }
        }
    }

    /**
     * sets up a player
     * @param _id
     *      player id
     * @param x
     *     x coordinate
     * @param y
     *     y coordinate
     * @param spriteNr
     *     sprite number in sprite list
     */
    function setupPlayer(_id, x, y, spriteNr) {
        if (!gameEngine.getPlayer(_id)) {
            gameEngine.setPlayer(
                new Player(
                    x, y,
                    ASSET_MANAGER.getAsset(playerSprites[spriteNr] + playerSpriteSuffix),
                    ASSET_MANAGER.getAsset(playerSprites[spriteNr] + playerLeftStr + playerSpriteSuffix),
                    ASSET_MANAGER.getAsset(playerSprites[spriteNr] + playerRightStr + playerSpriteSuffix)),
                _id);
        }
    }

    /**
     * updates a player
     * @param _id
     *      player id
     * @param x
     *     x coordinate
     * @param y
     *     y coordinate
     */
    function updatePlayer(_id, x, y, dl, dr) {
        if (id != _id) {
            var player = gameEngine.getPlayer(_id);
            if (player) {
                player.setX(x);
                player.setY(y);
                player.dirLeft = dl;
                player.dirRight = dr;
            }
        }
    }

    /**
     * updates the tiles
     * @param tiles
     *     list of all tiles
     */
    function updateTiles(tiles) {
        for (tile in tiles) {
            var t = tiles[tile];
            if (t) {
                if (!gameEngine.getTile(tile)) {
                    gameEngine.setTile(
                        new Tile(
                            tiles[tile].x, tiles[tile].y,
                            ASSET_MANAGER.getAsset(tileSprites[tiles[tile].spriteNr] + tileSpriteSuffix),
                            tiles[tile].plus),
                        tile);
                } else {
                    var pl = gameEngine.getTile(tile);
                    if (pl) {
                        pl.setX(t.x);
                        pl.setY(t.y);
                        pl.movingHL = t.movingHL;
                        pl.movingHR = t.movingHR;
                        pl.movingVU = t.movingVU;
                        pl.movingVD = t.movingVD;
                    }
                }
            }
        }
    }

    /**
     * updates the scores
     * @param scores
     *     list of scores
     */
    function updateScores(scores, activeClients) {
        var str = "<tr><th>" + str_players + " (<span id='" + id_clientCount + "'>" + activeClients + "</span>)</th><th>|</th><th>" + str_score + "</th></tr>";
        for (score in scores) {
            var name = (score == id) ? ("<span style='color:" + scoreNameColor + ";'>" + score.substr(0, 5) + "</span>") : score.substr(0, 5);
            str += "<tr><td>" + name + "</td><td>|</td><td>" + scores[score] + "</td></tr>";
        }
        $('#' + id_scoreTable).html(str);
        $("#" + id_clientCount).html(activeClients);
    }

    /**
     * initializes the canvas
     */
    function focusCanvas() {
        $("#" + id_canvas).attr("tabindex", "0"); // for the focus
        canvas.focus();
    }
    
    /**
     * (un-)highlights an id
     * @param id
     *      the id to (un-)highlight
     * @param bool
     *      true when highlight, false when un-highlight
     */
    function highlightId(id, bool) {
        $('#' + id).css({
            opacity: (bool ? 1.0 : 0.5)
        });
    }

    /**
     * sets the key events and own event handling
     */
    function setKeyEvents() {
        $("#canvas").keydown(function (e) {
            switch (e.which) {
            case 32:
                keySpace = true;
                highlightId(id_key_space, false);
                break;
            case 37:
                keyArrowLeft = true;
                highlightId(id_key_arrowLeft, false);
                break;
            case 39:
                keyArrowRight = true;
                highlightId(id_key_arrowRight, false);
                break;
            }
            e.preventDefault();
            return false;
        });

        $("#canvas").keyup(function (e) {
            switch (e.which) {
            case 32:
                keySpace = false;
                highlightId(id_key_space, true);
                break;
            case 37:
                highlightId(id_key_arrowLeft, true);
                keyArrowLeft = false;
                break;
            case 39:
                keyArrowRight = false;
                highlightId(id_key_arrowRight, true);
                break;
            }
            e.preventDefault();
            return false;
        });

        // check keys every n ms
        (function checkKeys() {
            if (keyArrowLeft) {
                gameEngine.getPlayer(id).left();
            }
            if (keyArrowRight) {
                gameEngine.getPlayer(id).right();
            }
            var pl = gameEngine.getPlayer(id);
            if (pl) {
                if (!firstRun && pl.posChanged()) {
                    socket.emit('playerUpdate', {
                        x: gameEngine.getPlayer(id).getX(),
                        y: gameEngine.getPlayer(id).getY(),
                        dl: gameEngine.getPlayer(id).dirLeft,
                        dr: gameEngine.getPlayer(id).dirRight
                    });
                }
                var kill = gameEngine.getKill();
                if (kill != false) {
                    socket.emit('playerKilled', {
                        id: kill
                    });
                    pl.killedAlready = true;
                }
            }
            gameEngine.keySpace = keySpace;
            setTimeout(checkKeys, keyTickRate);
        })();
    }

    /**
     * overwrites the submit handler
     */
    function submitHandler() {
        var msg = $('#' + id_newMessage).val().substr(0, 30);
        var findReplace = [
            [/&/g, "&amp;"],
            [/</g, "&lt;"],
            [/>/g, "&gt;"],
            [/"/g, "&quot;"]
        ]; //" <- only for one of my editors (TextMate) that does recognize the '/' in front of the quotation mark in the line above
        for (var item in findReplace) {
            msg = msg.replace(findReplace[item][0], findReplace[item][1]);
        }
        if (msg && msg != "") {
            socket.emit('chatUpdate', {
                msg: msg
            });
            $('#' + id_newMessage).val('');
        }
        focusCanvas();
        return false;
    }
    
    function prepareGame() {
        $(window).bind("submit", submitHandler);
        $('#' + id_div_loading).html(str_loading);
        $('#' + id_mainContainer + ', #' + id_div_loading).show();
        $('#' + id_main + ', #' + id_div_controls).show();
        $('#' + id_chat_window).html('<br />' + str_chatInit);

        queueDownloads();
        downloadAll();
    }

    // DOM ready
    $(document).ready(function () {
        $('#' + id_div_loading).html(str_firstStart);
        $('#' + id_mainContainer + ', #' + id_div_loading).show();
        $('#' + id_main + ', #' + id_div_controls).hide();
        $('#' + id_joinGame).click(function() {
            prepareGame();
            return false;
        });
    });
})(jQuery);

/**
 * class AssetManager
 */
function AssetManager() {
    this.successCount = 0;
    this.errorCount = 0;
    this.cache = {};
    this.downloadQueue = [];
}

/**
 * puts a new download to the queue
 * @param path
 *      path to the asset
 */
AssetManager.prototype.queueDownload = function (path) {
    this.downloadQueue.push(path);
}

/**
 * returns whether everything has been downloaded
 * @return
 *      true when downloads finished, false else
 */
AssetManager.prototype.isDone = function () {
    return (this.downloadQueue.length == this.successCount + this.errorCount);
}

/**
 * downloads all downloads in the queue and saves it in the cache
 * @param callback
 *      callback function when everything has been downloaded
 */
AssetManager.prototype.downloadAll = function (callback) {
    for (var i = 0; i < this.downloadQueue.length; ++i) {
        var path = this.downloadQueue[i];
        var img = new Image();
        var that = this;
        img.addEventListener("load", function () {
            that.successCount += 1;
            if (that.isDone()) {
                callback();
            }
        });
        img.addEventListener("error", function () {
            that.errorCount += 1;
            if (that.isDone()) {
                callback();
            }
        });
        img.src = path;
        this.cache[path] = img;
    }
}

/**
 * returns a downloaded asset
 * @param path
 *      path to the asset
 * @return
 *      a downloaded asset
 */
AssetManager.prototype.getAsset = function (path) {
    return this.cache[path];
}

/**
 * class Background
 */
function Background(sprite) {
    this.sprite = sprite;
}

/**
 * returns the sprite
 * @return
 *      the sprite
 */
Background.prototype.getSprite = function () {
    return this.sprite;
}

/**
 * sets the sprite
 * @param sprite
 *      the sprite
 */
Background.prototype.setSprite = function (sprite) {
    this.sprite = sprite;
}

/**
 * draws the background
 * @param context
 *      context to draw in
 */
Background.prototype.draw = function (context) {
    context.drawImage(this.sprite, 0, 0, context.canvas.width, context.canvas.height);
}

/**
 * class GameEngine
 */
function GameEngine(id) {
    this.id = id;
    this.background = null;
    this.tiles = [];
    this.players = {};
    this.context = null;
    this.lastUpdateTimestamp = 0;
    this.deltaTime = null;
    this.keySpace = false;
    this.kill = false;
    // message
    this.lastMsgTimestamp = null;
    this.msg = str_welcome;
    this.waitMsgIntv = 1500;
    this.msgAlreadyDrawn = false;
}

/**
 * constructor
 * @param context
 *      the context to render in
 */
GameEngine.prototype.init = function (context) {
    this.context = context;
}

/**
 * constructor
 * @param background
 *      the background
 */
GameEngine.prototype.setBackground = function (background) {
    this.background = background;
}

/**
 * sets up a tile
 * @param tile
 *      tile
 * @param position
 *      tile position in the container
 */
GameEngine.prototype.setTile = function (tile, position) {
    this.tiles[position] = tile;
}

/**
 * sets up a player
 * @param player
 *      player
 * @param id
 *      player id in the container
 */
GameEngine.prototype.setPlayer = function (player, id) {
    this.players[id] = player;
}

/**
 * displays a message
 * @param msg
 *      message to display
 */
GameEngine.prototype.setMessage = function (msg) {
    this.msg = msg;
    this.lastMsgTimestamp = Date.now();
    this.msgAlreadyDrawn = false;
}

/**
 * returns a tile
 * @param position
 *      tile position in the container
 */
GameEngine.prototype.getTile = function (position) {
    return this.tiles[position];
}

/**
 * returns a player
 * @param id
 *      player id in the container
 */
GameEngine.prototype.getPlayer = function (id) {
    return this.players[id];
}

/**
 * returns a list of all players
 * @return
 *      a list of all players
 */
GameEngine.prototype.getPlayers = function () {
    return this.players;
}

/**
 * draws a message when message not already displayed and timestamp ok
 */
GameEngine.prototype.drawMessage = function() {
    var now = Date.now();
    if(!this.msgAlreadyDrawn) {
        this.context.fillStyle = contextFillStyle;
        this.context.font = contextFont;
        this.context.textBaseline = 'top';
        this.context.fillText(this.msg, 0, 0);
    }
    if((now - this.lastMsgTimestamp) > this.waitMsgIntv) {
        this.msgAlreadyDrawn = true;
        this.lastMsgTimestamp = now;
    }
}

/**
 * clears the context
 */
GameEngine.prototype.clearContext = function () {
    // save custom transformation
    this.context.save();
    // set standard transformation
    this.context.setTransform(1, 0, 0, 1, 0, 0);
    // clear
    this.context.clearRect(0, 0, this.context.canvas.width, this.context.canvas.height);
    // draw background
    this.background.draw(this.context);
    // draw message
    this.drawMessage();
    // restore custom transformation
    this.context.restore();
}

/**
 * draws every entity and tile
 */
GameEngine.prototype.draw = function () {
    this.clearContext();
    // draw tiles
    for (var i = 0; i < this.tiles.length; ++i) {
        if(this.tiles[i]) {
            this.tiles[i].draw(this.context);
        }
    }
    // draw players
    for (player in this.players) {
        if(this.players[player]) {
            this.players[player].draw(this.context);
        }
    }
}

/**
 * returns the kills fo this player
 * @return
 *      an array with the kills (player IDs) of this player
 */
GameEngine.prototype.getKill = function () {
    return this.kill;
}

/**
 * checks the kills of this player
 */
GameEngine.prototype.checkKills = function () {
    this.kill = false;
    var pl1 = this.getPlayer(this.id);
    if (pl1 && !pl1.killedAlready) {
        for (pl2ID in this.players) {
            if (this.id != pl2ID) {
                var pl2 = this.getPlayer(pl2ID);
                if (pl2 && pl1.isAboveAndCollidesWithPlayer(pl2)) {
                    this.kill = pl2ID;
                    this.killedAlready = true;
                    break;
                }
            }
        }
    }
}

/**
 * updates every player and tile
 */
GameEngine.prototype.update = function () {
    var player = this.getPlayer(this.id);
    if (player) {
        player.setX(player.getX());
        player.setY(player.getY());
        var highestGround = 0;
        for (var j = 0; j < this.tiles.length; ++j) {
            var tile = this.tiles[j];
            if (!player.jumpUp && player.collidesWithTile(tile)) {
                highestGround = tile.getY() - tile.getSprite().height;
                if (tile.movingVU) {
                    player.setY(highestGround - tile.plus);
                }
                if (tile.movingHL) {
                    player.setX(player.getX() - tile.plus);
                } else if (tile.movingHR) {
                    player.setX(player.getX() + tile.plus);
                }
                break;
            } else if (player.isAboveTile(tile)) {
                var currGround = tile.getY() - tile.getSprite().height;
                if (currGround < highestGround) {
                    highestGround = currGround;
                }
            }
        }
        player.setCurrGround(highestGround);
        player.jump(this.keySpace);
        player.checkWhetherToJump();
    }
    this.checkKills();
}

/**
 * game loop
 */
GameEngine.prototype.loop = function () {
    this.update();
    this.draw();
    this.lastUpdateTimestamp = Date.now();
}

/**
 * starts the game
 */
GameEngine.prototype.start = function () {
    this.lastUpdateTimestamp = Date.now();
    this.lastMsgTimestamp = Date.now();
    var that = this;
    this.context.translate(0, this.context.canvas.height);
    (function gameloop() {
        that.loop();
        requestAnimFrame(gameloop, that.context.canvas);
    })();
}

/**
 * browser independent request animation frame
 */
window.requestAnimFrame = (function () {
    return  window.requestAnimationFrame ||
            window.webkitRequestAnimationFrame ||
            window.mozRequestAnimationFrame ||
            window.oRequestAnimationFrame ||
            window.msRequestAnimationFrame ||
            function (callback, element) {
                window.setTimeout(callback, 1000 / 60);
            };
})();

/**
 * checks whether num is in [lower, upper]
 * @param lower
 *      lower bound
 * @param upper
 *      upper bound
 * @param num
 *      number to check
 * @return
 *      true when num is in [lower, upper], false else
 */
function inIntervall(lower, upper, num) {
    if ((num >= lower) && (num <= upper)) {
        return true;
    }
    return false;
}
/**
 * class Player
 */
function Player(x, y, sprite, sprite_l, sprite_r, pathJumpSound, pathKillSound, pathHitSound) {
    this.x = x || 0;
    this.y = y || 0;
    this._x = x;
    this._y = y;
    this.currGround = 0;
    this.sprite = sprite;
    this.sprite_l = sprite_l;
    this.sprite_r = sprite_r;
    // "protected"
    this.plus = 5;
    this.jumpedFrom = this.currGround;
    this.isFalling = false;
    this.jumpUp = false;
    this.jumpDown = false;
    this.killedAlready = false;
    // "private"
    this.jumpMax = -145;
    this.currHeight = 0;
    this.jumpY = y;
    this.spaceKeyPressed = false;
    this.dirLeft = false;
    this.dirRight = false;
    this.pathJump = pathJumpSound || path_soundJump;
    this.pathKill = pathKillSound || path_soundKill;
    this.pathHit = pathHitSound || path_soundHit;
    // sounds
    this.soundJump = new Sound(this.pathKill);
    this.soundKill = new Sound(this.pathKill);
    this.soundHit = new Sound(this.pathHit);
}

// getter
Player.prototype.getX = function () {
    return this.x;
}

Player.prototype.getY = function () {
    return this.y;
}

Player.prototype.getCurrGround = function () {
    return this.currGround;
}

Player.prototype.getJumpMax = function () {
    return this.jumpMax;
}

Player.prototype.getSprite = function () {
    return this.sprite;
}

// getter without setter
Player.prototype.getPlus = function () {
    return this.plus;
}

Player.prototype.getJumpedFrom = function () {
    return this.jumpedFrom;
}

Player.prototype.getIsFalling = function () {
    return this.isFalling;
}

Player.prototype.getJumpUp = function () {
    return this.jumpUp;
}

Player.prototype.getJumpDown = function () {
    return this.jumpDown;
}

// setter
Player.prototype.setX = function (x) {
    this._x = this.x;
    this.x = x;
}

Player.prototype.setY = function (y) {
    this._y = this.y;
    this.y = y;
}

Player.prototype.setCurrGround = function (currGround) {
    this.currGround = currGround;
}

Player.prototype.setJumpMax = function (jMax) {
    this.jumpMax = jMax;
}

Player.prototype.setSprite = function (sprite) {
    this.sprite = sprite;
}

Player.prototype.posChanged = function () {
    return (this._x != this.x) || (this._y != this.y);
}

// sounds
Player.prototype.playJump = function () {
    var checked = $('#' + sound_id_jump).is(':checked');
    if (checked) {
        this.soundJump = new Audio(this.pathJump);
        if (this.soundJump) {
            this.soundJump.play();
        }
    }
}

Player.prototype.playKill = function () {
    var checked = $('#' + sound_id_kill).is(':checked');
    if (checked) {
        this.soundKill = new Sound(this.pathKill);
        if(this.soundKill) {
            this.soundKill.play();
        }
    }
}

Player.prototype.playHit = function () {
    var checked = $('#' + sound_id_hit).is(':checked');
    if (checked) {
        this.soundHit = new Sound(this.pathHit);
        if(this.soundHit) {
            this.soundHit.play();
        }
    }
}

// movement
Player.prototype.jump = function (spaceKeyPressed) {
    this.spaceKeyPressed = spaceKeyPressed;
    if (!(this.jumpUp || this.jumpDown || this.isFalling) && this.spaceKeyPressed) {
        this.playJump();
        this.jumpUp = true;
        this.jumpY = this.getY();
        this.jumpedFrom = this.getCurrGround();
    }
}

Player.prototype.up = function (plus) {
    plus = plus || this.plus;
    if (this.y > -windowHeight) {
        this.setY(this.y - plus);
    }
}

Player.prototype.down = function (plus) {
    plus = plus || this.plus;
    if (this.y < 0) {
        this.setY(this.y + plus);
    }
}

Player.prototype.left = function (plus) {
    this.dirLeft = true;
    this.dirRight = false;
    plus = plus || this.plus;
    if (this.x > 0 - (this.sprite.width / 2)) {
        this.setX(this.x - plus);
    } else {
        this.setX(windowWidth - this.sprite.width);
    }
}

Player.prototype.right = function (plus) {
    this.dirLeft = false;
    this.dirRight = true;
    plus = plus || this.plus;
    if (this.x < (windowWidth - this.sprite.width / 2)) {
        this.setX(this.x + plus);
    } else {
        this.setX(0);
    }
}

Player.prototype.checkWhetherToJump = function () {
    if (this.jumpUp && ((this.y - this.plus) > (this.jumpMax + this.jumpedFrom)) && this.spaceKeyPressed) {
        this.setY(this.y - this.plus);
        // dirty hack
    } else if (this.jumpUp && ((this.y - 4) > (this.jumpMax + this.jumpedFrom)) && this.spaceKeyPressed) {
        this.setY(this.y - 4);
    } else if (this.jumpUp && ((this.y - 3) > (this.jumpMax + this.jumpedFrom)) && this.spaceKeyPressed) {
        this.setY(this.y - 3);
    } else if (this.jumpUp && ((this.y - 2) > (this.jumpMax + this.jumpedFrom)) && this.spaceKeyPressed) {
        this.setY(this.y - 2);
    } else if (this.jumpUp && ((this.y - 1) > (this.jumpMax + this.jumpedFrom)) && this.spaceKeyPressed) {
        this.setY(this.y - 1);
        this.jumpUp = false;
        this.jumpDown = true;
    } else if (this.jumpUp) {
        this.jumpUp = false;
        this.jumpDown = true;
    } else if (this.jumpDown && ((this.y + this.plus) < this.currGround)) {
        this.setY(this.y + this.plus);
        // dirty hack
    } else if (this.jumpDown && ((this.y + 4) <= (this.currGround))) {
        this.setY(this.y + 4);
    } else if (this.jumpDown && ((this.y + 3) <= (this.currGround))) {
        this.setY(this.y + 3);
    } else if (this.jumpDown && ((this.y + 2) <= (this.currGround))) {
        this.setY(this.y + 2);
    } else if (this.jumpDown && ((this.y + 1) <= (this.currGround))) {
        this.setY(this.y + 1);
        this.jumpDown = false;
        this.killedAlready = false;
    } else if (this.jumpDown) {
        this.jumpDown = false;
        this.killedAlready = false;
    } else if ((this.y + this.plus) < this.currGround) {
        this.isFalling = true;
        this.setY(this.y + this.plus);
        // dirty hack
    } else if ((this.y + 4) <= this.currGround) {
        this.setY(this.y + 4);
    } else if ((this.y + 3) <= this.currGround) {
        this.setY(this.y + 3);
    } else if ((this.y + 2) <= this.currGround) {
        this.setY(this.y + 2);
    } else if ((this.y + 1) <= this.currGround) {
        this.setY(this.y + 1);
    } else {
        this.isFalling = false;
        this.killedAlready = false;
    }
}

// draw
Player.prototype.draw = function (context) {
    if (context) {
        var spr = this.dirLeft ? this.sprite_l : (this.dirRight ? this.sprite_r : this.sprite);
        context.drawImage(spr, this.getX(), this.getY() - this.getSprite().height, this.getSprite().width, this.getSprite().height);
        this.dirLeft = false;
        this.dirRight = false;
    }
}

// collision
Player.prototype.collidesWithTile = function (tile) {
    if (tile && ((this.getX() + this.getSprite().width) > tile.getX()) && (this.getX() < (tile.getX() + tile.getSprite().width)) && (inInterval(-this.plus, this.plus, this.y - (tile.getY() - tile.getSprite().height)))) {
        return true;
    }
    return false;
}

Player.prototype.isAboveAndCollidesWithPlayer = function (player) {
    if (player && (this.jumpDown || this.isFalling) && ((this.getX() + this.getSprite().width) > player.getX()) && (this.getX() < (player.getX() + player.getSprite().width)) && (inInterval(-this.plus, this.plus, this.y - (player.getY() - player.getSprite().height)))) {
        this.playKill();
        return true;
    }
    return false;
}

// misc
Player.prototype.isAboveTile = function (tile) {
    if (tile && ((this.getX() + this.getSprite().width) > tile.getX()) && (this.getX() < (tile.getX() + tile.getSprite().width)) && (this.y <= (tile.getY() - tile.getSprite().height))) {
        return true;
    }
    return false;
}

/**
 * class Sound
 */
function Sound(path) {
    this.path = path;
    this.sound = new Audio(this.path);
}

Sound.prototype.play = function () {
    if(this.sound) {
	    this.sound.play();
    }
}

Sound.prototype.stop = function () {
    if(this.sound) {
	    this.sound.pause();
	    this.sound = new Audio(this.path);
    }
}

/**
 * class Tile
 */
function Tile(x, y, sprite, plus) {
    this.x = x || 0;
    this.y = y || 0;
    this.plus = plus;
    this.sprite = sprite;
    this.movingHL = false;
    this.movingHR = false;
    this.movingVU = false;
    this.movingVD = false;
}

// getter
Tile.prototype.getX = function () {
    return this.x;
}

Tile.prototype.getY = function () {
    return this.y;
}

Tile.prototype.getSprite = function () {
    return this.sprite;
}

// setter
Tile.prototype.setX = function (x) {
    this.x = x;
}

Tile.prototype.setY = function (y) {
    this.y = y;
}

Tile.prototype.setSprite = function (sprite) {
    this.sprite = sprite;
}

// draw
Tile.prototype.draw = function (context) {
    context.drawImage(this.sprite, this.x, this.y - this.sprite.height, this.sprite.width, this.sprite.height);
}

